package com.stockr.server.news;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.client.RestClientTest;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.client.MockRestServiceServer;

import java.util.ArrayList;

import static org.junit.Assert.*;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.header;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.method;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.requestTo;
import static org.springframework.test.web.client.response.MockRestResponseCreators.withSuccess;

@RunWith(SpringRunner.class)
@RestClientTest(NewsService.class)
public class NewsServiceTest {
    @Autowired
    private NewsService ns;

    @Autowired
    private MockRestServiceServer mrss;

    @Autowired
    private ObjectMapper om;

    private String JSON;

    @Before
    public void setUp() throws JsonProcessingException {
        setUpJsonResponse();
        setUpMrss();
    }

    private void setUpMrss() {
        mrss.expect(requestTo(ns.insertQueryInUrl("test")))
                .andExpect(method(HttpMethod.GET))
                .andExpect(header("X-Api-Key", Matchers.containsString("")))
                .andRespond(withSuccess(JSON, MediaType.APPLICATION_JSON));
    }

    private void setUpJsonResponse() throws JsonProcessingException {
        ArrayList<Article> al = new ArrayList<>();
        Article a = new Article();
        a.setAuthor("Author Name");
        al.add(a);
        News n = new News();
        n.setArticles(al);
        JSON = om.writeValueAsString(n);
    }

    @Test
    public void getNews_ShouldHitTheProperEndpoint() {
        ns.getNews("test");
        mrss.verify();
    }

    @Test
    public void getNewsShouldSendXApiKeyHeader() {
        ns.getNews("test");
        mrss.verify();
    }

    @Test
    public void getNews_ShouldParseJsonProperly() {
        News ne = ns.getNews("test");
        assertEquals("Author Name", ne.getArticles().get(0).getAuthor());
    }

    @Test
    public void getAllArticles_ShouldReturnArrayList() {
        ArrayList ne = ns.getAllArticles("test");
        assertNotNull(ne);
    }


    @Test
    public void insertQueryInUrl_ShouldPassQueryIntoUrl() {
        String query = "tesla";
        String expected = "https://newsapi.org/v2/everything?q=" + query +
                "&language=en" +
                "&sources=bloomberg,business-insider,cnbc,the-wall-street-journal" +
                "&pageSize=100";
        String actual = ns.insertQueryInUrl(query);
        assertEquals(expected, actual);
    }
}
